@@ -207,7 +207,7 @@ public class RecordReaderImpl implements RecordReader {
     OrcFile.WriterVersion writerVersion = fileReader.getWriterVersion();
     SchemaEvolution evolution;
     if (options.getSchema() == null) {
-      LOG.info("Reader schema not provided -- using file schema " +
+      LOG.debug("Reader schema not provided -- using file schema " +
           fileReader.getSchema());
       evolution = new SchemaEvolution(fileReader.getSchema(), null, options);
     } else {
@@ -667,6 +667,14 @@ public class RecordReaderImpl implements RecordReader {
                                            TypeDescription type,
                                            boolean writerUsedProlepticGregorian,
                                            boolean useUTCTimestamp) {
+
+    // When statsProto is EMPTY_COLUMN_STATISTICS
+    // this column does not actually provide statistics
+    // we cannot make any assumptions, so we return YES_NO_NULL.
+    if (statsProto == EMPTY_COLUMN_STATISTICS) {
+      return TruthValue.YES_NO_NULL;
+    }
+
     ColumnStatistics cs = ColumnStatisticsImpl.deserialize(
         null, statsProto, writerUsedProlepticGregorian, true);
     ValueRange range = getValueRange(cs, predicate, useUTCTimestamp);
@@ -748,21 +756,16 @@ public class RecordReaderImpl implements RecordReader {
                                            ValueRange range,
                                            BloomFilter bloomFilter,
                                            boolean useUTCTimestamp) {
+    // If range is invalid, that means that no value (including null) is written to this column
+    // we should return TruthValue.NO for any predicate.
     if (!range.isValid()) {
-      return TruthValue.YES_NO_NULL;
+      return TruthValue.NO;
     }
 
     // if we didn't have any values, everything must have been null
     if (!range.hasValues()) {
       if (predicate.getOperator() == PredicateLeaf.Operator.IS_NULL) {
         return TruthValue.YES;
-      } else if (predicate.getOperator() == PredicateLeaf.Operator.NULL_SAFE_EQUALS) {
-        Object literal = predicate.getLiteral();
-        if (literal == null) {
-          return TruthValue.YES;
-        } else {
-          return TruthValue.NO;
-        }
       } else {
         return TruthValue.NULL;
       }
@@ -790,11 +793,11 @@ public class RecordReaderImpl implements RecordReader {
     // 1) Bloom filter is available
     // 2) Min/Max evaluation yield YES or MAYBE
     // 3) Predicate is EQUALS or IN list
-    return bloomFilter != null
-           && result != TruthValue.NO_NULL && result != TruthValue.NO
-           && (predicate.getOperator().equals(PredicateLeaf.Operator.EQUALS)
-               || predicate.getOperator().equals(PredicateLeaf.Operator.NULL_SAFE_EQUALS)
-               || predicate.getOperator().equals(PredicateLeaf.Operator.IN));
+    return bloomFilter != null &&
+           result != TruthValue.NO_NULL && result != TruthValue.NO &&
+           (predicate.getOperator().equals(PredicateLeaf.Operator.EQUALS) ||
+               predicate.getOperator().equals(PredicateLeaf.Operator.NULL_SAFE_EQUALS) ||
+               predicate.getOperator().equals(PredicateLeaf.Operator.IN));
   }
 
   private static TruthValue evaluatePredicateMinMax(PredicateLeaf predicate,
@@ -1183,7 +1186,8 @@ public class RecordReaderImpl implements RecordReader {
             leafValues[pred] = exceptionAnswer[pred];
           } else {
             if (indexes[columnIx] == null) {
-              throw new AssertionError("Index is not populated for " + columnIx);
+              LOG.warn("Index is not populated for " + columnIx);
+              return READ_ALL_RGS;
             }
             OrcProto.RowIndexEntry entry = indexes[columnIx].getEntry(rowGroup);
             if (entry == null) {
@@ -1221,8 +1225,8 @@ public class RecordReaderImpl implements RecordReader {
                   LOG.warn(reason, e);
                 }
                 boolean hasNoNull = stats.hasHasNull() && !stats.getHasNull();
-                if (predicate.getOperator().equals(PredicateLeaf.Operator.NULL_SAFE_EQUALS)
-                    || hasNoNull) {
+                if (predicate.getOperator().equals(PredicateLeaf.Operator.NULL_SAFE_EQUALS) ||
+                    hasNoNull) {
                   exceptionAnswer[pred] = TruthValue.YES_NO;
                 } else {
                   exceptionAnswer[pred] = TruthValue.YES_NO_NULL;
@@ -1535,9 +1539,9 @@ public class RecordReaderImpl implements RecordReader {
     // within strip). Batch size computed out of marker position makes sure that batch size is
     // aware of row group boundary and will not cause overflow when reading rows
     // illustration of this case is here https://issues.apache.org/jira/browse/HIVE-6287
-    if (rowIndexStride != 0
-        && (includedRowGroups != null || startReadPhase != TypeReader.ReadPhase.ALL)
-        && rowInStripe < rowCountInStripe) {
+    if (rowIndexStride != 0 &&
+        (includedRowGroups != null || startReadPhase != TypeReader.ReadPhase.ALL) &&
+        rowInStripe < rowCountInStripe) {
       int startRowGroup = (int) (rowInStripe / rowIndexStride);
       if (includedRowGroups != null && !includedRowGroups[startRowGroup]) {
         while (startRowGroup < includedRowGroups.length && !includedRowGroups[startRowGroup]) {
@@ -1549,8 +1553,7 @@ public class RecordReaderImpl implements RecordReader {
       // We force row group boundaries when dealing with filters. We adjust the end row group to
       // be the next row group even if more than one are possible selections.
       if (includedRowGroups != null && startReadPhase == TypeReader.ReadPhase.ALL) {
-        while (endRowGroup < includedRowGroups.length
-               && includedRowGroups[endRowGroup]) {
+        while (endRowGroup < includedRowGroups.length && includedRowGroups[endRowGroup]) {
           endRowGroup += 1;
         }
       } else {
@@ -1610,8 +1613,8 @@ public class RecordReaderImpl implements RecordReader {
                                boolean[] included,
                                boolean[] readCols) throws IOException {
     // Use the cached objects if the read request matches the cached request
-    if (stripeIndex == currentStripe
-            && (readCols == null || Arrays.equals(readCols, rowIndexColsToRead))) {
+    if (stripeIndex == currentStripe &&
+        (readCols == null || Arrays.equals(readCols, rowIndexColsToRead))) {
       if (rowIndexColsToRead != null) {
         return indexes;
       } else {
